/*
 * Copyright (C) 2009 by Matthias Ringwald
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MATTHIAS RINGWALD AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL MATTHIAS
 * RINGWALD OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#pragma once

#import <Foundation/Foundation.h>
#import <stdint.h>
#import <btstack/btstack.h>

#define PREFS_REMOTE_NAME  @"RemoteName"
#define PREFS_LINK_KEY     @"LinkKey"
#define BTstackManagerID   @"ch.ringwald.btstack"

@class BTDevice;

/*
 * Information on devices is stored in a system-wide plist
 * it is maintained by BTstackManager
 * this includes the link keys
 */

// TODO enumerate BTstackError type
typedef int BTstackError;

typedef enum {
	kDeactivated = 1,
	kW4SysBTState,
	kW4SysBTDisabled,
	kW4Activated,
	kActivated,
	kW4Deactivated,
	kSleeping,
#if 1
	kW4DisoveryStopped,
	kW4AuthenticationEnableCommand
#endif
} ManagerState;

typedef enum {
	kInactive = 1,
	kW4InquiryMode,
	kInquiry,
	kRemoteName,
	// stopping
	kW4InquiryModeBeforeStop,
	kW4InquiryStop,
	kW4RemoteNameBeforeStop,
} DiscoveryState;

@protocol BTstackManagerDelegate;
@protocol BTstackManagerListener;

@interface BTstackManager : NSObject {
@private
	NSObject<BTstackManagerDelegate>* _delegate;
	NSMutableArray *discoveredDevices;
	BOOL connectedToDaemon;
	ManagerState state;
	DiscoveryState discoveryState;
	int discoveryDeviceIndex;
    
	// current connection - kind a ugly
	uint8_t   connType; // 0 = L2CAP, 1 = RFCOMM
	bd_addr_t connAddr;
	uint16_t  connPSM;
	uint16_t  connChan;
	uint8_t   connAuth;
}

// shared instance
+(BTstackManager *) sharedInstance;

// Activation
-(BTstackError) activate;
-(BTstackError) deactivate;
-(BOOL) isActivating;
-(BOOL) isActive;

// Discovery
-(BTstackError) startDiscovery;
-(BTstackError) stopDiscovery;
-(int) numberOfDevicesFound;
-(BTDevice*) deviceAtIndex:(int)index;
-(BOOL) isDiscoveryActive;

// Link Key Management
-(void) dropLinkKeyForAddress:(bd_addr_t*) address;

-(BTstackError) createRFCOMMConnection:(bd_addr_t*)addr onChannel:(uint16_t)rfcomm_channel;
-(BTstackError) sendRFCOMMPacket:(uint8_t*)packet ofSize:(uint16_t)size ForChannel:(uint16_t)channel;


// TODO add l2cap and rfcomm incoming commands
@property (nonatomic, assign) NSObject<BTstackManagerDelegate>* delegate;
@property (nonatomic, retain) NSMutableArray *discoveredDevices;
@end


@protocol BTstackManagerDelegate
@optional

// Activation callbacks
-(BOOL) disableSystemBluetoothBTstackManager:(BTstackManager*) manager; // default: YES

// Connection events
-(NSString*) btstackManager:(BTstackManager*) manager pinForAddress:(bd_addr_t)addr; // default: "0000"

// direct access
-(void) btstackManager:(BTstackManager*) manager
  handlePacketWithType:(uint8_t) packet_type
			forChannel:(uint16_t) channel
			   andData:(uint8_t *)packet
			   withLen:(uint16_t) size;


// Activation events
-(void) activatedBTstackManager:(BTstackManager*) manager;
-(void) btstackManager:(BTstackManager*)manager activationFailed:(BTstackError)error;
-(void) deactivatedBTstackManager:(BTstackManager*) manager;

// Discovery events: general
-(void) btstackManager:(BTstackManager*)manager deviceInfo:(BTDevice*)device;

-(void) btstackManager:(BTstackManager*)manager rfcommDataReceivedFromChannel:(uint16_t)channel withData:(uint8_t *)packet ofLen:(uint16_t)size;
-(void) btstackManager:(BTstackManager*)manager rfcommChannelOpened:(uint16_t)channel mtu:(uint16_t)mtu;
-(void) btstackManager:(BTstackManager*)manager rfcommConnexionFailedWithStatus:(uint8_t)status;

@end
